body {
  font-family: "Corbel";
  font-size: 1.16em;
}

h1 {
  font-weight: bold;
  font-size: 24pt;
  color: #660033;
}

h2 {
  font-weight: bold;
  font-size: 16pt;
  color: #FF6633;
}

a {
  color: #FF784B;
}
a:hover {
  color: #FF784B;
}

.section.metadata {
  font-size: 12pt;
  color: #999999;
}


/* main template */

#tools-group, #pagefooter, .panel, table, .breadcrumb, .nav-tabs {
  font-size: 1.08em;
}


/* banner */

.container {
  max-width: 966px;
}

#logo {
  height: 174px;
  width: 174px;
  background-image: url("logo_saem.png");
}

#tools-group {
  background: url("bandeau_saem.jpg") no-repeat;
}

.navbar-default {
  background-color: #FF6633;
}

.navbar-default .navbar-nav li {
  padding: 0 1px;
  margin: 0 1px;
  width: 152px;
}

.navbar-default .navbar-left {
  padding-top: 105px;
}

.navbar-default .navbar-left > li > a {
  color: #660033;
  background-color: #CCCCCC;
  padding: 5px 15px 10px 15px;
  border-top-left-radius: 8px;
  border-top-right-radius: 8px;
}

.navbar-default .navbar-left > li > a:hover {
  background-color: #DDDDDD;
}

.navbar-default .navbar-left > li.active > a {
  background-color: #660033;
  color: #FFFFFF;
}

#search_box {
  margin-top: 10px;
  display: inline;
}

.pull-right.dropdown {
  margin-top: 10px;
  margin-right: 5px;
  display: inline;
}

.pull-right.dropdown .dropdown-menu {
  left: auto;
  right: 0;
}

.pull-right.dropdown a {
 color: #777;
}

.pull-right .open > a,
.pull-right.dropdown a:hover {
 color: #333;
}

/* breadcrumbs */

.cw-breadcrumb {
  background-color: #F5F5F5;
}

ul.breadcrumb {
  padding-left: 0px;
}

#breadcrumbs a {
  color: #660033;
}

#breadcrumbs a:hover {
  text-decoration: underline;
}


/* forms */

div.formGroup {
  border: 1px solid #BBB;
  padding-left: 3px;
  padding-right: 3px;
}

.form-control.metaField, .form-control.hasDatepicker {
  width: 90%;
  display: inline;
}

.form-control.metaField.hasDatepicker{
  width: 86%;
}


/* index page */

.panel .list-group .list-group-item {
  padding: .2em 1em;
}

.panel-info {
  border-color: #757575;
}

.panel-info .panel-heading {
  color: #FFFFFF;
  background-color: #757575;
}

.panel-info .panel-heading a {
  color: inherit;
}

.panel-info .panel-heading a:hover {
  color: #FF784B;
  text-decoration: none;
}

.panel-default .panel-heading {
  font-weight: bold;
  background-color: #CCCCCC;
}


/* other custom classes */

div.facet div.facetTitle {
  font-weight: normal;
  background-color: #CCCCCC;
}

.similar_values-category {
  font-weight: bold;
  padding: .2em .4em;
  margin: .8em 0 .2em;
  line-height: 1.5;
 }

.similar_values-list {
  background: #F5F5F5;
}

.highlight {
  background-color: #F6A048;
}

.jOrgChart {
  margin-top: 3em;
}

.jOrgChart .node {
  height: auto !important;
}